
#include "stdafx.h"

#include "ManagedLock.h"
#include "DataQueue.h"
#include "Tuner.h"

namespace Bon
{
	DataQueue::DataQueue(Tuner^ parent, int maxQueueUnits)
		: parent_(parent), maxQueueUnits_(maxQueueUnits),
		queue_(gcnew System::Collections::Generic::Queue<System::Collections::Generic::KeyValuePair<array<System::Byte>^, int> >()),
		queued_(gcnew System::Threading::ManualResetEvent(false))
	{
	}

	DataQueue::~DataQueue()
	{
		if (parent_ != nullptr)
		{
			parent_->RemoveQueue(this);
			parent_ = nullptr;
		}

		Close();
	}

	void DataQueue::Close()
	{
		Internal::ManagedLock lock(queue_);

		queue_->Clear();
		queued_->Set();
	}

	void DataQueue::Clear()
	{
		Internal::ManagedLock lock(queue_);

		queue_->Clear();
		if (parent_ != nullptr)
		{
			queued_->Reset();
		}
	}

	System::Threading::WaitHandle^ DataQueue::CanDequeueHandle::get()
	{
		return queued_;
	}

	void DataQueue::Enqueue(System::Collections::Generic::KeyValuePair<array<System::Byte>^, int> data)
	{
		Internal::ManagedLock lock(queue_);

		queue_->Enqueue(data);
		if (queue_->Count == 1)
		{
			queued_->Set();
		}
	}

	System::Collections::Generic::KeyValuePair<array<System::Byte>^, int> DataQueue::Dequeue()
	{
		queued_->WaitOne();

		Internal::ManagedLock lock(queue_);

		if (queue_->Count == 0)
		{
			throw gcnew System::ObjectDisposedException("DataQueue disposed");
		}

		System::Collections::Generic::KeyValuePair<array<System::Byte>^, int> data(queue_->Dequeue());
		if (queue_->Count == 0)
		{
			queued_->Reset();
		}

		return data;
	}
}
