
#include "stdafx.h"

#include "BonException.h"
#include "ManagedLock.h"
#include "DataQueue.h"
#include "DataQueueManager.h"
#include "Tuner.h"

namespace Bon
{
	namespace Internal
	{
		template <typename T> T *CreateBonInstance(LPCTSTR pClassName)
		{
			T *pInstance = BON_SAFE_CREATE<T *>(pClassName, 0);
			if (pInstance == 0)
			{
				throw gcnew BonException("Cannot create instance: Name={0}",
					gcnew System::String(pClassName));
			}
			return pInstance;
		}
	}

	void Tuner::RemoveQueue(DataQueue^ queue)
	{
		Internal::ManagedLock lock(this);

		int count = pManager_->RemoveQueue(queue);
		if (count == 0)
		{
			if (pTsSourceTuner_ != nullptr)
			{
				dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->StopDecoder();
				pTsDescrambler_->EnableEmmProcess(false);
				pTsSourceTuner_->GetHalTsTuner()->SetLnbPower(false);
			}
		}
	}

	Tuner::Tuner(System::String^ tunerName)
		: pTsSourceTuner_(0), pTsPacketSync_(0), pProgAnalyzer_(0), pTsDescrambler_(0), pManager_(0)
	{
		pTsSourceTuner_ = Internal::CreateBonInstance<ITsSourceTuner>(TEXT("CTsSourceTuner"));
		pTsPacketSync_ = Internal::CreateBonInstance<ITsPacketSync>(TEXT("CTsPacketSync"));
		pProgAnalyzer_ = Internal::CreateBonInstance<IProgAnalyzer>(TEXT("CProgAnalyzer"));
		pTsDescrambler_ = Internal::CreateBonInstance<ITsDescrambler>(TEXT("CTsDescrambler"));
		pManager_ = new Internal::DataQueueManager();

		dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->SetOutputDecoder(dynamic_cast<IMediaDecoder *>(pTsPacketSync_));
		dynamic_cast<IMediaDecoder *>(pTsPacketSync_)->SetOutputDecoder(dynamic_cast<IMediaDecoder *>(pTsDescrambler_));
		dynamic_cast<IMediaDecoder *>(pTsDescrambler_)->SetOutputDecoder(dynamic_cast<IMediaDecoder *>(pProgAnalyzer_));
		dynamic_cast<IMediaDecoder *>(pProgAnalyzer_)->SetOutputDecoder(pManager_);

		pin_ptr<const wchar_t> pTunerName = PtrToStringChars(tunerName);

		int tunerResult = pTsSourceTuner_->OpenTuner(pTunerName);
		if (tunerResult > 0)
		{
			throw gcnew BonException("Cannot open tuner: Name={0}, Result={1}",
				tunerName, tunerResult);
		}

		if (pTsDescrambler_->OpenDescrambler(TEXT("CBcasCardReader")) == false)
		{
			pTsSourceTuner_->CloseTuner();
			throw gcnew BonException("Cannot open card reader");
		}

		pTsDescrambler_->DiscardScramblePacket(false);
		pTsDescrambler_->EnableBuffering(false);
	}

	Tuner::~Tuner()
	{
		Internal::ManagedLock lock(this);

		BON_SAFE_RELEASE(pTsSourceTuner_);
		BON_SAFE_RELEASE(pTsPacketSync_);
		BON_SAFE_RELEASE(pTsDescrambler_);
		BON_SAFE_RELEASE(pProgAnalyzer_);
		BON_SAFE_DELETE(pManager_);
	}

	DataQueue^ Tuner::CreateQueue()
	{
		Internal::ManagedLock lock(this);

		System::Collections::Generic::KeyValuePair<DataQueue^, int> queue =
			pManager_->CreateQueue(this);
		if (queue.Value == 1)
		{
			pTsDescrambler_->EnableEmmProcess(false);
			pTsSourceTuner_->GetHalTsTuner()->SetLnbPower(true);
			pTsSourceTuner_->SetChannel(0UL, 9UL);			// ND4 110CS #2
			dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->PlayDecoder();
		}
		return queue.Key;
	}

	void Tuner::SetChannel(int space, int channel)
	{
		Internal::ManagedLock lock(this);

		pTsSourceTuner_->SetChannel(space, channel);
	}
}
