// SmartThread.cpp: X}[gXbhNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "SmartThread.h"
#include <TChar.h>
#include <process.h>


namespace Bon
{

/////////////////////////////////////////////////////////////////////////////
// tOVOiNX
/////////////////////////////////////////////////////////////////////////////

CFlagSignal::CFlagSignal(DWORD dwResolution)
	: m_dwResolution(max(dwResolution, 1)), m_bFlag(false)
{
}

void CFlagSignal::Release()
{
	delete this;
}

const IKernelObject *CFlagSignal::GetKernelObject() const
{
	return 0;
}

const bool CFlagSignal::Set()
{
	m_bFlag = true;
	return true;
}

const bool CFlagSignal::Reset()
{
	m_bFlag = false;
	return true;
}

CFlagSignal::operator bool() const
{
	return m_bFlag;
}

bool CFlagSignal::operator !() const
{
	return m_bFlag == false;
}

const IWaitableObject::WaitResult CFlagSignal::Wait(const DWORD dwTimeout) const
{
	switch (dwTimeout)
	{
	case 0:
		if (m_bFlag == true)
		{
			return Signal0;
		}
		break;

	case INFINITE:
		while (m_bFlag == false)
		{
			::Sleep(m_dwResolution);
		}
		return Signal0;

	default:
		// ̂悤ȂȂ̂ŁAmł͂Ȃ
		for (DWORD time = 0; time < dwTimeout; time += m_dwResolution)
		{
			if (m_bFlag == true)
			{
				return Signal0;
			}
			::Sleep(m_dwResolution);
		}
		break;
	}

	return Timeout;
}


/////////////////////////////////////////////////////////////////////////////
// CxgVOiNX
/////////////////////////////////////////////////////////////////////////////

CEventSignal::CEventSignal()
	: m_Event(true)
{
}

void CEventSignal::Release()
{
	delete this;
}

const IKernelObject *CEventSignal::GetKernelObject() const
{
	return &m_Event;
}

const bool CEventSignal::Set()
{
	return m_Event.SetEvent();
}

const bool CEventSignal::Reset()
{
	return m_Event.ResetEvent();
}

CEventSignal::operator bool() const
{
	return m_Event.Wait(0) == Signal0;
}

bool CEventSignal::operator !() const
{
	return m_Event.Wait(0) != Signal0;
}

const IWaitableObject::WaitResult CEventSignal::Wait(const DWORD dwTimeout) const
{
	return m_Event.Wait(dwTimeout);
}


/////////////////////////////////////////////////////////////////////////////
// X}[gXbhNX
/////////////////////////////////////////////////////////////////////////////

CSmartThreadBase::CSmartThreadBase(ISignalInstance *pKillSignal)
	: m_pThreadParam(NULL)
	, m_dwThreadID(0UL)
	, m_pKillSignal(pKillSignal)
	, m_SyncSignal(true)
{
}

CSmartThreadBase::~CSmartThreadBase(void)
{
	EndThread();
	BON_SAFE_RELEASE(m_pKillSignal);
}

const bool CSmartThreadBase::SetPriority(int nPriority)
{
	BOOL result = ::SetThreadPriority(UnsafeGetHandle(), nPriority);
	return result == TRUE;
}

const bool CSmartThreadBase::EndThread(const bool bPumpMessage)
{
	if(!IsAssigned())return false;

	// IVOiZbg
	m_pKillSignal->Set();

	// Xbh̒Ăяoꍇ͊JȂ
	if(::GetCurrentThreadId() == m_dwThreadID)return true;

	// TXyhԂ̂Ƃ̓W[
	::ResumeThread(UnsafeGetHandle());

	// I҂
	if(!bPumpMessage)
	{
		Wait();
	}
	else
	{
		while (Wait(0) == Timeout)
		{
			PumpMessage();
		}
	}

	// nh
	Close();
	m_dwThreadID = 0UL;

	return true;	
}

const bool CSmartThreadBase::SuspendThread(void)
{
	if(!IsAssigned())false;

	// XbhTXyh
	::SuspendThread(UnsafeGetHandle());

	return true;	
}

const bool CSmartThreadBase::ResumeThread(void)
{
	if(!IsAssigned())false;

	// XbhW[
	::ResumeThread(UnsafeGetHandle());

	return true;	
}

const DWORD CSmartThreadBase::GetThreadID(void) const
{
	// XbhIDԂ
	return m_dwThreadID;
}

const bool CSmartThreadBase::IsRunning(void) const
{
	// XbhsLԂ
	return (m_SyncSignal.Wait(0) == Signal0) && (Wait(0) == Timeout);
}

const bool CSmartThreadBase::StartThread(PVOID pThreadParam, const bool bStart)
{
	// JĂȂꍇ͊J
	EndThread();

	// p[^ۑ
	m_pThreadParam = pThreadParam;
	
	// VOiNA
	m_SyncSignal.ResetEvent();
	m_pKillSignal->Reset();
	
	// XbhN
	unsigned int threadID = 0;
	HANDLE hThread = reinterpret_cast<HANDLE>(_beginthreadex(
		NULL, 0UL, CSmartThreadBase::ThreadFunc, static_cast<void *>(this), (bStart)? 0UL : CREATE_SUSPENDED, &threadID));
	if(!hThread)
	{
		// XbhNs
		m_pThreadParam = 0;
		return false;
	}

	// XbhN҂
	if(bStart)
	{
		m_SyncSignal.Wait();
	}	

	Assign(hThread);
	m_dwThreadID = threadID;

	return true;
}

void CSmartThreadBase::PumpMessage(void)
{
	MSG Msg;

	// bZ[W|v쓮
	while(::PeekMessage(&Msg, NULL, 0UL, 0UL, PM_REMOVE)){
		::TranslateMessage(&Msg);
		::DispatchMessage(&Msg);
		}
}

unsigned int WINAPI CSmartThreadBase::ThreadFunc(void *pParam)
{
	CSmartThreadBase *pThis = static_cast<CSmartThreadBase *>(pParam);

	// TXyhԂŏI
	if(*(pThis->m_pKillSignal)){
		pThis->m_SyncSignal.ResetEvent();
		return 0UL;	
		}

	// NVOiZbg
	pThis->m_SyncSignal.SetEvent();

	try
	{
		// Xbh\bhs
		pThis->ThreadEntry(*(pThis->m_pKillSignal), pThis->m_pThreadParam);
	}
	catch (...)
	{
#ifdef _DEBUG
		TCHAR szBuffer[128];
		::_stprintf(szBuffer, TEXT("Caught unknown exception in thread %d\r\n"), pThis->m_dwThreadID);

		::OutputDebugString(szBuffer);
#endif
	}

	// NVOiNA
	pThis->m_SyncSignal.ResetEvent();

	return 0UL;	
}

}
