// SyncObject.cpp: IuWFNgbp[NX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "SyncObject.h"
#include <TChar.h>
#include <assert.h>


namespace Bon
{

//////////////////////////////////////////////////////////////////////
// NeBJZNVbp[NX
//////////////////////////////////////////////////////////////////////

CSmartLock::CSmartLock()
{
	// NeBJZNV
	::InitializeCriticalSection(&m_CriticalSection);
}

CSmartLock::~CSmartLock()
{
	// NeBJZNV폜
	::DeleteCriticalSection(&m_CriticalSection);
}

void CSmartLock::Lock(void)
{
	// NeBJZNV擾
	::EnterCriticalSection(&m_CriticalSection);
}

void CSmartLock::Unlock(void)
{
	// NeBJZNVJ
	::LeaveCriticalSection(&m_CriticalSection);
}
	
	
//////////////////////////////////////////////////////////////////////
// ubNXR[vbNNX
//////////////////////////////////////////////////////////////////////

CBlockLock::CBlockLock(CSmartLock *pSmartLock)
	: m_pSmartLock(pSmartLock)
{
	// bN擾
	if(m_pSmartLock)m_pSmartLock->Lock();
}

CBlockLock::~CBlockLock()
{
	// bNJ
	if(m_pSmartLock)m_pSmartLock->Unlock();
}


/////////////////////////////////////////////////////////////////////////////
// J[lIuWFNgANX
/////////////////////////////////////////////////////////////////////////////

CWaitObjectsBase::CWaitObjectsBase(const IKernelObject &rhs)
	: m_dwCount(0)
{
	if (rhs.IsAssigned() == true)
	{
		m_Handles[0] = rhs.UnsafeGetHandle();
		m_dwCount = 1;
	}
}

CWaitObjectsBase::CWaitObjectsBase(const CWaitObjectsBase &rhs)
	: m_dwCount(rhs.m_dwCount)
{
	memcpy(m_Handles, rhs.m_Handles, sizeof m_Handles);
}

void CWaitObjectsBase::Copy(const CWaitObjectsBase &rhs)
{
	m_dwCount = rhs.m_dwCount;
	memcpy(m_Handles, rhs.m_Handles, sizeof m_Handles);
}

void CWaitObjectsBase::Append(const IKernelObject &rhs)
{
	if (m_dwCount >= MAXIMUM_WAIT_OBJECTS)
	{
		assert(0);
		return;
	}

	if (rhs.IsAssigned() == true)
	{
		HANDLE handle = rhs.UnsafeGetHandle();
		m_Handles[m_dwCount] = handle;
		m_dwCount++;
	}
}

void CWaitObjectsBase::Append(const IKernelObject *pRhs)
{
	if (m_dwCount >= MAXIMUM_WAIT_OBJECTS)
	{
		assert(0);
		return;
	}

	if (pRhs->IsAssigned() == true)
	{
		HANDLE handle = pRhs->UnsafeGetHandle();
		m_Handles[m_dwCount] = handle;
		m_dwCount++;
	}
}

const IWaitableObject::WaitResult CWaitObjectsBase::Wait(bool waitAll, const DWORD dwTimeout) const
{
	return static_cast<IWaitableObject::WaitResult>(
		WaitForMultipleObjects(m_dwCount, m_Handles, waitAll ? TRUE : FALSE, dwTimeout));
}


/////////////////////////////////////////////////////////////////////////////
// J[lIuWFNgNX
/////////////////////////////////////////////////////////////////////////////

CKernelObject::CKernelObject(HANDLE handle, HANDLE invalid)
	: m_Handle(handle), m_Invalid(invalid)
{
}

CKernelObject::~CKernelObject()
{
	Close();
}

void CKernelObject::Assign(HANDLE handle)
{
	Close();
	m_Handle = handle;
}

const bool CKernelObject::Close()
{
	if (m_Handle == m_Invalid)
	{
		return TRUE;
	}

	BOOL result = CloseHandle(m_Handle);
	m_Handle = m_Invalid;
	return result == TRUE;
}

const bool CKernelObject::IsAssigned() const
{
	return m_Handle != m_Invalid;
}

const HANDLE CKernelObject::UnsafeGetHandle() const
{
	return m_Handle;
}

CKernelObject::operator const bool() const
{
	return m_Handle != m_Invalid;
}

const bool CKernelObject::operator !() const
{
	return m_Handle == m_Invalid;
}

CWaitAnyObjects CKernelObject::operator ||(const IKernelObject &rhs) const
{
	return CWaitAnyObjects(*this) || rhs;
}

CWaitAnyObjects CKernelObject::operator ||(const IKernelObject *pRhs) const
{
	return CWaitAnyObjects(*this) || *pRhs;
}

CWaitAllObjects CKernelObject::operator &&(const IKernelObject &rhs) const
{
	return CWaitAllObjects(*this) && rhs;
}

CWaitAllObjects CKernelObject::operator &&(const IKernelObject *pRhs) const
{
	return CWaitAllObjects(*this) && *pRhs;
}

const IWaitableObject::WaitResult CKernelObject::Wait(const DWORD dwTimeout) const
{
	return static_cast<WaitResult>(WaitForSingleObject(m_Handle, dwTimeout));
}


//////////////////////////////////////////////////////////////////////
// Cxgbp[NX
//////////////////////////////////////////////////////////////////////

CSmartEvent::CSmartEvent(const bool bManualReset, const bool bInitialState)
	: CKernelObject(::CreateEvent(NULL, (bManualReset)? TRUE : FALSE, (bInitialState)? TRUE : FALSE, NULL))
{
}

const bool CSmartEvent::Close(void)
{
	return CKernelObject::Close();
}

const bool CSmartEvent::SetEvent(void)
{
	// CxgZbg
	return (::SetEvent(UnsafeGetHandle()))? true : false;
}

const bool CSmartEvent::PulseEvent(void)
{
	// CxgZbg(1̃Xbĥ݊J)
	return (::PulseEvent(UnsafeGetHandle()))? true : false;
}
const bool CSmartEvent::ResetEvent(void)
{
	// CxgZbg
	return (::ResetEvent(UnsafeGetHandle()))? true : false;
}


/////////////////////////////////////////////////////////////////////////////
// ~[ebNXbp[NX
/////////////////////////////////////////////////////////////////////////////

CSmartMutex::CSmartMutex(void)
{
}

const bool CSmartMutex::Create(LPCTSTR lpszName)
{
	// ɂ͔Ή(vZXԂ̔r̂)
	if(!lpszName)return false;
	if(!::_tcslen(lpszName))return false;

	// ɃI[vς݂̏ꍇ̓G[
	if(IsAssigned())return false;
	
	// Mutex쐬
	HANDLE hMutex = ::CreateMutex(NULL, TRUE, lpszName);
	if(!hMutex)return false;
	
	// ̃vZXŃI[vĂꍇ̓G[
	if(::GetLastError() == ERROR_ALREADY_EXISTS){
		return false;
		}

	Assign(hMutex);

	return true;
}

const bool CSmartMutex::Close(void)
{
	return CKernelObject::Close();
}

const bool CSmartMutex::IsExist(LPCTSTR lpszName)
{
	// w肵OMutexI[v
	HANDLE hMutex = ::OpenMutex(MUTEX_ALL_ACCESS, FALSE, lpszName);
	
	if(hMutex){
		// Mutex݂͑
		::CloseHandle(hMutex);
		return true;
		}
	else{
		// Mutex݂͑Ȃ
		return false;
		}	
}

}
