using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Threading;
using System.IO;

namespace Bon
{
	public partial class MainForm : Form
	{
		private HttpListener listener_ = new HttpListener();
		private Tuner tuner_;

		public MainForm()
		{
			InitializeComponent();

			tuner_ = new Tuner("CBonBlackTuner");

			listener_.Prefixes.Add("http://+:80/");
			listener_.Start();

			listener_.BeginGetContext(new AsyncCallback(AcceptEntry), null);
		}

		private void AcceptEntry(IAsyncResult ar)
		{
			try
			{
				HttpListenerContext context = listener_.EndGetContext(ar);
				listener_.BeginGetContext(new AsyncCallback(AcceptEntry), null);

				DateTime now = DateTime.Now;
				TreeNode node = new TreeNode(String.Format("{0} [{1:yyyy/MM/dd HH:mm:ss}]",
					context.Request.RemoteEndPoint, now));
				node.Nodes.Add("---");
				node.Nodes.Add("0 Bytes sent");
				node.Tag = context;
				this.Invoke(new ParameterizedThreadStart(BeginContext), node);

				Thread thread = new Thread(new ParameterizedThreadStart(DistributeEntry));
				thread.Start(node);
			}
			catch
			{
			}
		}

		private void DistributeEntry(object argument)
		{
			TreeNode node = (TreeNode)argument;
			HttpListenerContext context = (HttpListenerContext)node.Tag;

			context.Response.ContentType = "video/mpeg";

			try
			{
				long sent = 0;
				DateTime latest = DateTime.Now;

				using (Stream stream = context.Response.OutputStream)
				{
					using (DataQueue queue = tuner_.CreateQueue())
					{
						while (true)
						{
							KeyValuePair<byte[], int> data = queue.Dequeue();
							stream.Write(data.Key, 0, data.Value);

							sent += data.Value;

							DateTime now = DateTime.Now;
							TimeSpan span = now - latest;
							if (span.TotalSeconds > 2.0)
							{
								latest = now;
								this.BeginInvoke(new ParameterizedThreadStart(UpdateContext),
									new KeyValuePair<TreeNode, long>(node, sent));
							}
						}
					}
				}
			}
			catch
			{
			}
			finally
			{
				try
				{
					context.Response.Close();
				}
				catch
				{
				}
				try
				{
					if (this.IsDisposed == false)
					{
						this.Invoke(new ParameterizedThreadStart(FinishContext), node);
					}
				}
				catch
				{
				}
			}
		}

		private void BeginContext(object argument)
		{
			TreeNode node = (TreeNode)argument;
			HttpListenerContext context = (HttpListenerContext)node.Tag;

			try
			{
				treeView1.Nodes.Add(node);
				node.ExpandAll();
			}
			catch
			{
			}
		}

		private void UpdateContext(object argument)
		{
			KeyValuePair<TreeNode, long> pair = (KeyValuePair<TreeNode, long>)argument;
			TreeNode node = pair.Key;
			HttpListenerContext context = (HttpListenerContext)pair.Key.Tag;
			long sent = (long)pair.Value;

			try
			{
				node.Nodes[0].Text = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss");
				node.Nodes[1].Text = string.Format("{0} Bytes sent", sent);
			}
			catch
			{
			}
		}

		private void FinishContext(object argument)
		{
			TreeNode node = (TreeNode)argument;
			HttpListenerContext context = (HttpListenerContext)node.Tag;

			try
			{
				treeView1.Nodes.Remove(node);
			}
			catch
			{
			}
		}

		private void MainForm_Load(object sender, EventArgs e)
		{

		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			listener_.Stop();
			tuner_.Dispose();
		}

		private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
		}
	}
}
