// SmartThread.h: X}[gXbhNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


namespace Bon
{

/////////////////////////////////////////////////////////////////////////////
// VOiC^[tFCX
/////////////////////////////////////////////////////////////////////////////

class ISignalInstance : public ISignal
{
public:
	virtual void Release() = 0;
};

/////////////////////////////////////////////////////////////////////////////
// tOVOiNX
/////////////////////////////////////////////////////////////////////////////

class CFlagSignal : public ISignalInstance
{
public:
	CFlagSignal(DWORD dwResolution = 100);

	const IKernelObject *GetKernelObject() const;

	const bool Set();
	const bool Reset();

	operator bool() const;
	bool operator !() const;

	const WaitResult Wait(const DWORD dwTimeout = INFINITE) const;

	void Release();

private:
	const DWORD m_dwResolution;
	bool m_bFlag;
};


/////////////////////////////////////////////////////////////////////////////
// CxgVOiNX
/////////////////////////////////////////////////////////////////////////////

class CEventSignal : public ISignalInstance
{
public:
	CEventSignal();

	const IKernelObject *GetKernelObject() const;

	const bool Set();
	const bool Reset();

	operator bool() const;
	bool operator !() const;

	const WaitResult Wait(const DWORD dwTimeout = INFINITE) const;

	void Release();

private:
	CSmartEvent m_Event;
};


/////////////////////////////////////////////////////////////////////////////
// X}[gXbhNX
/////////////////////////////////////////////////////////////////////////////

class CSmartThreadBase : public CKernelObject
{
public:
	~CSmartThreadBase(void);

	const bool SetPriority(int nPriority);
	const bool EndThread(const bool bPumpMessage = false);

	const bool SuspendThread(void);
	const bool ResumeThread(void);

	const DWORD GetThreadID(void) const;
	const bool IsRunning(void) const;
	
protected:
	CSmartThreadBase(ISignalInstance *pKillSignal);

	const bool StartThread(PVOID pThreadParam, const bool bStart);

	static void PumpMessage(void);

	virtual void ThreadEntry(const ISignal &killSignal, void *pThreadParam) = 0;

private:
	static unsigned int WINAPI ThreadFunc(void *pParam);

	void *m_pThreadParam;
	DWORD m_dwThreadID;
	CSmartEvent m_SyncSignal;
	ISignalInstance *m_pKillSignal;
};


/////////////////////////////////////////////////////////////////////////////
// X}[gXbhNX
/////////////////////////////////////////////////////////////////////////////

template <class T> class CSmartThread : public CSmartThreadBase
{
public:
	// Xbh\bh^
	typedef void (T::*TAGET_FUNC)(CSmartThread<T> &thread, const ISignal &killSignal, PVOID pParam);

	CSmartThread(ISignalInstance *pKillSignal = new CEventSignal())
		: CSmartThreadBase(pKillSignal)
		, m_pThreadOwner(NULL)
		, m_pTargetFunc(NULL)
	{
	}

	const bool StartThread(T *pOwner, TAGET_FUNC pTargetFunc, PVOID pThreadParam = NULL, const bool bStart = true)
	{
		if(!pOwner || !pTargetFunc)return false;
		
		// ֐|C^Ap[^ۑ
		m_pThreadOwner = pOwner;
		m_pTargetFunc = pTargetFunc;
		
		return CSmartThreadBase::StartThread(pThreadParam, bStart);
	}
	
protected:
	void ThreadEntry(const ISignal &killSignal, void *pThreadParam)
	{
		// Xbh\bhs
		(m_pThreadOwner->*m_pTargetFunc)(*this, killSignal, pThreadParam);
	}

private:
	T *m_pThreadOwner;
	TAGET_FUNC m_pTargetFunc;
};

}
