// TsSourceTuner.cpp: TS\[X`[ifR[_
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "TsSourceTuner.h"


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

#define TSBUFFER_SIZE	0x10000UL	// 64KB


/////////////////////////////////////////////////////////////////////////////
// TS\[X`[ifR[_
/////////////////////////////////////////////////////////////////////////////

const DWORD CTsSourceTuner::OpenTuner(LPCTSTR lpszBCId)
{
	// UN[Y
	CloseTuner();

	IHalDevice *pHalDevice = NULL;

	try{
		// `[ĩCX^X𐶐
		if(!(pHalDevice = ::BON_SAFE_CREATE<IHalDevice *>(lpszBCId)))throw (DWORD)EC_INTERNAL_ERR;

		// foCX^Cv`FbN
		if(pHalDevice->GetDeviceType() != ::BON_NAME_TO_GUID(TEXT("IHalTsTuner")))throw (DWORD)EC_INTERNAL_ERR;

		// foCX݂̑`FbN
		if(!pHalDevice->GetTotalDeviceNum())throw (DWORD)EC_DEVICE_NONE;

		// foCX̋󂫂`FbN
		if(pHalDevice->GetActiveDeviceNum() >= pHalDevice->GetTotalDeviceNum())throw (DWORD)EC_DEVICE_FULL;
	
		// IHalTsTunerC^tF[X擾
		if(!(m_pHalTsTuner = dynamic_cast<IHalTsTuner *>(pHalDevice)))throw (DWORD)EC_INTERNAL_ERR;

		// TSobt@CX^X
		if(!(m_pTsBuffer = ::BON_SAFE_CREATE<IMediaData *>(TEXT("CMediaData"))))throw (DWORD)EC_INTERNAL_ERR;
	
		// TSobt@TCYm
		if(m_pTsBuffer->GetBuffer(TSBUFFER_SIZE) != TSBUFFER_SIZE)throw (DWORD)EC_INTERNAL_ERR;

		// `[iI[v
		if(!m_pHalTsTuner->OpenTuner())throw (DWORD)EC_OPEN_FAILED;

		// TSMXbhN
		if(!m_TsRecvThread.StartThread(this, &CTsSourceTuner::TsRecvThread))throw (DWORD)EC_INTERNAL_ERR;
		}
	catch(DWORD dwErrCode){
		// G[
		BON_SAFE_RELEASE(pHalDevice);
		m_pHalTsTuner = NULL;
		
		CloseTuner();
		return dwErrCode;
		}

	return EC_OPEN_SUCCESS;
}

const bool CTsSourceTuner::CloseTuner(void)
{
	// TSMXbh~
	if(m_bIsPlaying)StopDecoder();

	m_bIsPlaying = false;
	m_TsRecvThread.EndThread(true);

	// `[iN[Y
	if(m_pHalTsTuner)m_pHalTsTuner->CloseTuner();

	// TSobt@J
	BON_SAFE_RELEASE(m_pTsBuffer);

	// `[iCX^XJ
	BON_SAFE_RELEASE(m_pHalTsTuner);
	
	return true;
}

const DWORD CTsSourceTuner::GetDeviceName(LPTSTR lpszName)
{
	// IHalDeviceC^tF[X擾
	IHalDevice * const pHalDevice = dynamic_cast<IHalDevice *>(m_pHalTsTuner);

	// foCXԂ
	return (pHalDevice)? pHalDevice->GetDeviceName(lpszName) : 0UL;
}

const DWORD CTsSourceTuner::GetTuningSpaceName(const DWORD dwSpace, LPTSTR lpszName)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return 0UL;

	// `[jOԖԂ
	return m_pHalTsTuner->EnumTuningSpace(dwSpace, lpszName);
}

const DWORD CTsSourceTuner::GetChannelName(const DWORD dwSpace, const DWORD dwChannel, LPTSTR lpszName)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return 0UL;

	// `lԂ
	return m_pHalTsTuner->EnumChannelName(dwSpace, dwChannel, lpszName);
}

const DWORD CTsSourceTuner::GetCurSpace(void)
{
	// ݂̃`[jOԂԂ
	return (m_pHalTsTuner)? m_pHalTsTuner->GetCurSpace() : 0UL;
}

const DWORD CTsSourceTuner::GetCurChannel(void)
{
	// ݂̃`lԂ
	return (m_pHalTsTuner)? m_pHalTsTuner->GetCurChannel() : 0UL;
}

const bool CTsSourceTuner::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return 0UL;

	// `lύX
	if(!m_pHalTsTuner->SetChannel(dwSpace, dwChannel))return false;
	
	// ʃfR[_Zbg
	ResetDecoder();

	return true;
}

const float CTsSourceTuner::GetSignalLevel(void)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return 0UL;

	// MxԂ
	return m_pHalTsTuner->GetSignalLevel();
}

IHalTsTuner * CTsSourceTuner::GetHalTsTuner(void)
{
	// `[ifoCXԂ
	return m_pHalTsTuner;
}

CTsSourceTuner::CTsSourceTuner(IBonObject *pOwner)
	: CMediaDecoder(pOwner, 0UL, 1UL, new CSmartLock)
	, m_pHalTsTuner(NULL)
	, m_pTsBuffer(NULL)
	, m_bIsPlaying(false)
{

}

CTsSourceTuner::~CTsSourceTuner(void)
{
	CloseTuner();
}

const bool CTsSourceTuner::OnPlay(void)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return false;

	// łɍĐ
	if(m_bIsPlaying)return true;

	// ̃Xg[j
	m_pHalTsTuner->PurgeStream();

	// Xg[ĐԂɂ
	m_bIsPlaying = true;

	return true;
}

const bool CTsSourceTuner::OnStop(void)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return false;

	// Xg[~Ԃɂ
	m_bIsPlaying = false;
	
	return true;
}

const bool CTsSourceTuner::OnReset(void)
{
	// ̃Xg[j
	if(m_bIsPlaying && m_pHalTsTuner)m_pHalTsTuner->PurgeStream();

	return true;
}

void CTsSourceTuner::TsRecvThread(CSmartThread<CTsSourceTuner> &thread, const ISignal &bKillSignal, PVOID pParam)
{
	// `[iTSf[^oXbh
	while (true)
	{
		IWaitableObject::WaitResult result =
			(*bKillSignal.GetKernelObject() || m_pHalTsTuner->GetStreamEvent()).Wait();
		if (result != IWaitableObject::Signal1)
		{
			break;
		}

		DWORD dwRemains = 0UL;
		do
		{
			BYTE *pStreamData = NULL;
			DWORD dwStreamSize = 0UL;
			if (m_pHalTsTuner->GetStream(&pStreamData, &dwStreamSize, &dwRemains))
			{
				if (pStreamData && dwStreamSize)
				{
					OnTsStream(pStreamData, dwStreamSize);
				}
			}
		}
		while (dwRemains > 0UL);

		Sleep(1UL);
	}
}

const bool CTsSourceTuner::PurgeStream(void)
{
	// `[iI[vĂȂ
	if(!m_pHalTsTuner)return false;

	// ̃Xg[j
	m_pHalTsTuner->PurgeStream();

	// ʃfR[_Zbg
	ResetDecoder();

	return true;
}

void CTsSourceTuner::OnTsStream(BYTE *pStreamData, DWORD dwStreamSize)
{
	CBlockLock AutoLock(m_pLock);

	// ~͉Ȃ
	if(!m_bIsPlaying)return;

	// f[^obt@ɃZbg
	m_pTsBuffer->SetData(pStreamData, dwStreamSize);

	// ̃fR[_Ƀf[^n
	OutputMedia(m_pTsBuffer);
}
