// PtBaseTuner.cpp: PT`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "PtBaseTuner.h"
#include <TChar.h>


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

// foCX
#define DEVICENAME_TER	TEXT("Earthsoft PT1 ISDB-T fW^`[i")
#define DEVICENAME_SAT	TEXT("Earthsoft PT1 ISDB-S fW^`[i")

// obt@ݒ
#define FIFOBUFFTIME	2UL														// obt@ = 2b
#define FIFOBLOCKSIZE	(CPtBoard::DMABLK_SIZE / CPtBoard::DMACHK_UNIT / 3UL)	// ubNTCY
#define FIFOBUFFNUM		(0x3C0000UL / FIFOBLOCKSIZE * FIFOBUFFTIME)				// 30MbpsƂ


/////////////////////////////////////////////////////////////////////////////
// PT1`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

const bool CPtBaseTunerProperty::LoadProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;

	// ݒǂݏo
	m_bDefaultLnbPower = pStorage->GetBoolItem(TEXT("DefaultLnbPower"	), false);
	m_bEnaleVHF		   = pStorage->GetBoolItem(TEXT("EnaleVHF"			), true	);
	m_bEnaleCATV	   = pStorage->GetBoolItem(TEXT("EnaleCATV"			), true	);
	m_bEnaleCS		   = pStorage->GetBoolItem(TEXT("EnaleCS"			), true	);
	m_bEnaleBS		   = pStorage->GetBoolItem(TEXT("EnaleBS"			), true	);

	return true;
}

const bool CPtBaseTunerProperty::SaveProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;
	
	// ݒۑ
	try{
		if(!pStorage->SetBoolItem(TEXT("DefaultLnbPower"), m_bDefaultLnbPower	))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleVHF"		), m_bEnaleVHF			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleCATV"		), m_bEnaleCATV			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleCS"		), m_bEnaleCS			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleBS"		), m_bEnaleBS			))throw ::BON_EXPECTION();
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		return false;
		}
	
	return true;
}

IConfigTarget * CPtBaseTunerProperty::GetConfigTarget(void)
{
	// RtBO^[QbgԂ
	return m_pConfigTarget;
}

const bool CPtBaseTunerProperty::CopyProperty(const IConfigProperty *pProperty)
{
	const IPtBaseTunerProperty *pPtBaseTunerProperty = dynamic_cast<const IPtBaseTunerProperty *>(pProperty);
	if(!pPtBaseTunerProperty)return false;
	
	// vpeBRs[
	*(static_cast<IPtBaseTunerProperty *>(this)) = *pPtBaseTunerProperty;
	
	return true;
}

const DWORD CPtBaseTunerProperty::GetDialogClassName(LPTSTR lpszClassName)
{
	// vpeB_CAO͔
	return 0UL;
}

CPtBaseTunerProperty::CPtBaseTunerProperty(IBonObject *pOwner)
	: CBonObject(pOwner)
	, m_pConfigTarget(dynamic_cast<IConfigTarget *>(pOwner))
{
	// ftHglݒ
	m_bDefaultLnbPower = false;
	m_bEnaleVHF = true;
	m_bEnaleCATV = true;
	m_bEnaleCS = true;
	m_bEnaleBS = true;
}

CPtBaseTunerProperty::~CPtBaseTunerProperty(void)
{
	// Ȃ
}


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

CPtManager *CPtBaseTuner::m_pPtManager = NULL;
DWORD CPtBaseTuner::m_dwInstanceCount = 0UL;

const BONGUID CPtBaseTuner::GetDeviceType(void)
{
	// foCX̃^CvԂ
	return ::BON_NAME_TO_GUID(TEXT("IHalTsTuner"));
}

const DWORD CPtBaseTuner::GetDeviceName(LPTSTR lpszName)
{
	static const TCHAR aszDeviceName[][256] = {DEVICENAME_TER, DEVICENAME_SAT};

	// foCXԂ
	if(lpszName)::_tcscpy(lpszName, aszDeviceName[m_dwTunerType]);

	return ::_tcslen(aszDeviceName[m_dwTunerType]);
}

const DWORD CPtBaseTuner::GetTotalDeviceNum(void)
{
	// `[ȋԂ
	return m_pPtManager->GetTotalTunerNum(m_dwTunerType);
}

const DWORD CPtBaseTuner::GetActiveDeviceNum(void)
{
	// gp̃`[iԂ
	return m_pPtManager->GetActiveTunerNum(m_dwTunerType);
}

const bool CPtBaseTuner::OpenTuner(void)
{
	// UN[Y
	CloseTuner();
	
	try{
		// `[iI[v
		if(!(m_pTuner = m_pPtManager->OpenTuner(m_dwTunerType)))throw ::BON_EXPECTION(TEXT("`[iI[vs"));

		// LNBdݒ
		if(m_dwTunerType == Device::ISDB_S){
			if(!SetLnbPower(m_Property.m_bDefaultLnbPower))throw ::BON_EXPECTION(TEXT("LNBdݒ莸s"));
			}

		// ftHg`lݒ
		SetChannel(m_dwCurSpace, m_dwCurChannel);

		// FIFOobt@m
		m_BlockPool.resize(FIFOBUFFNUM, CMediaData(FIFOBLOCKSIZE));
		m_itFreeBlock = m_BlockPool.begin();

		// Xg[MJn
		if(!m_pTuner->StartStream(this))throw ::BON_EXPECTION(TEXT("TSXg[MJns"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		CloseTuner();
		return false;
		}
	
	return true;
}

void CPtBaseTuner::CloseTuner(void)
{
	// `[iN[Y
	if(m_pTuner){
		m_pTuner->StopStream();
		m_pTuner->CloseTuner();
		m_pTuner = NULL;
		}
	
	// FIFOobt@J
	{
		CBlockLock AutoLock(&m_FifoLock);
	
		while(!m_FifoBuffer.empty())m_FifoBuffer.pop();
		m_BlockPool.clear();
	}
}

const DWORD CPtBaseTuner::EnumTuningSpace(const DWORD dwSpace, LPTSTR lpszSpace)
{
	// gp\ȃ`[jOԂԂ
	if(dwSpace >= m_TuningSpace.size())return 0UL;

	// `[jOԖRs[
	if(lpszSpace)::_tcscpy(lpszSpace, m_TuningSpace[dwSpace]->szSpaceName);

	// `[jOԖԂ
	return ::_tcslen(m_TuningSpace[dwSpace]->szSpaceName);
}

const DWORD CPtBaseTuner::EnumChannelName(const DWORD dwSpace, const DWORD dwChannel, LPTSTR lpszChannel)
{
	// gp\ȃ`lԂ
	if(dwSpace >= m_TuningSpace.size())return 0UL;

	if(dwChannel >= m_TuningSpace[dwSpace]->dwChNum)return 0UL;

	// `lRs[
	if(lpszChannel)::_tcscpy(lpszChannel, m_TuningSpace[dwSpace]->pChConfig[dwChannel].szChName);

	// `lԂ
	return ::_tcslen(m_TuningSpace[dwSpace]->pChConfig[dwChannel].szChName);
}

const DWORD CPtBaseTuner::GetCurSpace(void)
{
	// ݂̃`[jOԂԂ
	return m_dwCurSpace;
}

const DWORD CPtBaseTuner::GetCurChannel(void)
{
	// ݂̃`lԂ
	return m_dwCurChannel;
}

const bool CPtBaseTuner::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	// ȂAhNXŃI[o[ChȂ΂ȂȂ
	::BON_ASSERT(false);
	
	return false;
}

const bool CPtBaseTuner::SetLnbPower(const bool bEnable)
{
	// LNBd
	return (m_pTuner)? m_pTuner->SetLnbPower(bEnable) : false;
}

const float CPtBaseTuner::GetSignalLevel(void)
{
	// CNR擾
	return (m_pTuner)? m_pTuner->GetSignalLevel() : 0.0f;
}

const bool CPtBaseTuner::GetStream(BYTE **ppStream, DWORD *pdwSize, DWORD *pdwRemain)
{
	CBlockLock AutoLock(&m_FifoLock);

	// MFIFOobt@Xg[o
	if(!m_pTuner)
	{
		// `[iI[vĂȂ
		return false;
	}
	else if(m_FifoBuffer.empty())
	{
		// o\ȃf[^Ȃ
		if(ppStream)*ppStream = NULL;
		if(pdwSize)*pdwSize = 0UL;
		if(pdwRemain)*pdwRemain = 0UL;
	}
	else
	{	
		// f[^o
		if(ppStream)*ppStream = m_FifoBuffer.front()->GetData();
		if(pdwSize)*pdwSize = m_FifoBuffer.front()->GetSize();

		m_FifoBuffer.pop();
		
		if(pdwRemain)*pdwRemain = m_FifoBuffer.size();

		if(m_FifoBuffer.size() == 0)
		{
			// CxgZbg
			m_FifoEvent.ResetEvent();
		}
	}

	return true;
}

void CPtBaseTuner::PurgeStream(void)
{
	// FIFOobt@NA
	CBlockLock AutoLock(&m_FifoLock);
	
	while(!m_FifoBuffer.empty())m_FifoBuffer.pop();
}

const IKernelObject &CPtBaseTuner::GetStreamEvent() const
{
	// Xg[EFCg
	return m_FifoEvent;
}

const DWORD CPtBaseTuner::GetAvailableNum(void)
{
	// o\Xg[Ԃ
	return m_FifoBuffer.size();
}

const bool CPtBaseTuner::SetProperty(const IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeBݒ
	return m_Property.CopyProperty(pProperty);
}

const bool CPtBaseTuner::GetProperty(IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeB擾
	return pProperty->CopyProperty(&m_Property);
}

const DWORD CPtBaseTuner::GetPropertyClassName(LPTSTR lpszClassName)
{
	static const TCHAR szPropertyClassName[] = TEXT("CPtBaseTunerProperty");

	// vpeBNX擾
	if(lpszClassName)::_tcscpy(lpszClassName, szPropertyClassName);

	return ::_tcslen(szPropertyClassName);
}

CPtBaseTuner::CPtBaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType)
	: CBonObject(pOwner)
	, m_Property(static_cast<IConfigTarget *>(this))
	, m_pTuner(NULL)
	, m_dwTunerType(dwTunerType)
	, m_dwCurSpace(0UL)
	, m_dwCurChannel(0UL)
	, m_FifoEvent(true)
{
	// }l[WCX^X
	if(!m_dwInstanceCount++){
		m_pPtManager = new CPtManager;
		}

	::BON_ASSERT(m_pPtManager != NULL);

	// ftHgvpeBf
	SetProperty(&m_Property);
}

CPtBaseTuner::~CPtBaseTuner(void)
{
	CloseTuner();
	
	// }l[WCX^XJ
	if(!--m_dwInstanceCount){
		BON_SAFE_DELETE(m_pPtManager);
		}
}

void CPtBaseTuner::OnTsStream(CPtTuner *pTuner, const BYTE *pData, const DWORD dwSize)
{
	CBlockLock AutoLock(&m_FifoLock);
	
	if(m_FifoBuffer.size() < (FIFOBUFFNUM - 1UL))
	{
		// FIFOɓ
		m_itFreeBlock->SetData(pData, dwSize);
		m_FifoBuffer.push(&(*m_itFreeBlock));
		if(++m_itFreeBlock == m_BlockPool.end())m_itFreeBlock = m_BlockPool.begin();
		
		if(m_FifoBuffer.size() == 1)
		{
			// CxgZbg
			m_FifoEvent.SetEvent();
		}
	}
	else
	{
		::BON_TRACE(TEXT("FIFOobt@I[o[t[\n"));
	}
}
